@@ -2,16 +2,16 @@
 Copyright (c) 2012, Brice Videau <brice.videau@imag.fr>
 Copyright (c) 2012, Vincent Danjean <Vincent.Danjean@ens-lyon.org>
 All rights reserved.
-      
+
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
-    
+
 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
-        
+
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -56,8 +56,8 @@ int debug_ocl_icd_mask=0;
 typedef __typeof__(clGetPlatformInfo) *clGetPlatformInfo_fn;
 
 inline void dump_vendor_icd(const char* info, const struct vendor_icd *v) {
-  debug(D_DUMP, "%s %p={ num=%i, first=%i, handle=%p, f=%p}\n", info,
-	v, v->num_platforms, v->first_platform, v->dl_handle, v->ext_fn_ptr);
+  debug(D_DUMP, "%s %p={ num=%i, handle=%p, f=%p}\n", info,
+	v, v->num_platforms, v->dl_handle, v->ext_fn_ptr);
 }
 
 struct vendor_icd *_icds=NULL;
@@ -213,7 +213,7 @@ static inline unsigned int _open_driver(unsigned int num_icds,
   if (f==NULL) {
     RETURN(num_icds);
   }
-  
+
   fseek(f, 0, SEEK_END);
   lib_path_length = ftell(f)+1;
   fseek(f, 0, SEEK_SET);
@@ -232,7 +232,7 @@ static inline unsigned int _open_driver(unsigned int num_icds,
   }
 
   lib_path_length = strnlen(lib_path, lib_path_length);
-  
+
   if( lib_path[lib_path_length-1] == '\n' )
     lib_path[lib_path_length-1] = '\0';
 
@@ -270,7 +270,7 @@ static void* _get_function_addr(void* dlh, clGetExtensionFunctionAddress_fn fn,
     }
 #ifdef DEBUG_OCL_ICD
     if (addr1 && addr2 && addr1!=addr2) {
-      debug(D_WARN, "Function and symbol '%s' have different addresses!", name);
+      debug(D_WARN, "Function and symbol '%s' have different addresses (%p != %p)!", name, addr2, addr1);
     }
 #endif
   }
@@ -319,6 +319,71 @@ static char* _malloc_clGetPlatformInfo(clGetPlatformInfo_fn plt_info_ptr,
   RETURN_STR(param_value);
 }
 
+static void _count_devices(struct platform_icd *p) {
+  cl_int error;
+
+  /* Ensure they are 0 in case of errors */
+  p->ngpus = p->ncpus = p->ndevs = 0;
+
+  error = clGetDeviceIDs(p->pid, CL_DEVICE_TYPE_GPU, 0, NULL, &(p->ngpus));
+  if (error != CL_SUCCESS && error != CL_DEVICE_NOT_FOUND){
+    debug(D_WARN, "Error %s while counting GPU devices in platform %p",
+	  _clerror2string(error), p->pid);
+  }
+
+  error = clGetDeviceIDs(p->pid, CL_DEVICE_TYPE_CPU, 0, NULL, &(p->ncpus));
+  if (error != CL_SUCCESS && error != CL_DEVICE_NOT_FOUND){
+    debug(D_WARN, "Error %s while counting CPU devices in platform %p",
+	  _clerror2string(error), p->pid);
+  }
+
+  error = clGetDeviceIDs(p->pid, CL_DEVICE_TYPE_ALL, 0, NULL, &(p->ndevs));
+  if (error != CL_SUCCESS && error != CL_DEVICE_NOT_FOUND){
+    debug(D_WARN, "Error %s while counting ALL devices in platform %p",
+	  _clerror2string(error), p->pid);
+  }
+
+}
+
+static int _cmp_platforms(const void *_a, const void *_b) {
+	const struct platform_icd *a=(const struct platform_icd *)_a;
+	const struct platform_icd *b=(const struct platform_icd *)_b;
+
+	/* sort first platforms handling max gpu */
+	if (a->ngpus > b->ngpus) return -1;
+	if (a->ngpus < b->ngpus) return 1;
+	/* sort next platforms handling max cpu */
+	if (a->ncpus > b->ncpus) return -1;
+	if (a->ncpus < b->ncpus) return 1;
+	/* sort then platforms handling max devices */
+	if (a->ndevs > b->ndevs) return -1;
+	if (a->ndevs < b->ndevs) return 1;
+	/* else consider platforms equal */
+	return 0;
+}
+
+static void _sort_platforms(struct platform_icd *picds, int npicds) {
+	debug(D_WARN, "Nb platefroms: %i", npicds);
+	if (npicds > 1) {
+		char* ocl_sort=getenv("OCL_ICD_PLATFORM_SORT");
+		if (ocl_sort!=NULL && !strcmp(ocl_sort, "none")) {
+			debug(D_LOG, "Platform not sorted");
+		} else {
+			if (ocl_sort!=NULL && strcmp(ocl_sort, "devices")) {
+				debug(D_WARN, "Unknown platform sort algorithm requested: %s", ocl_sort);
+				debug(D_WARN, "Switching do the 'devices' algorithm");
+			}
+			int i;
+			debug(D_LOG, "Platform sorted by GPU, CPU, DEV");
+			for (i=0; i<npicds; i++) {
+				_count_devices(&picds[i]);
+			}
+			qsort(picds, npicds, sizeof(*picds),
+			      &_cmp_platforms);
+		}
+	}
+}
+
 static inline void _find_and_check_platforms(cl_uint num_icds) {
   cl_uint i;
   _num_icds = 0;
@@ -328,9 +393,9 @@ static inline void _find_and_check_platforms(cl_uint num_icds) {
     struct vendor_icd *picd = &_icds[i];
     void* dlh = _icds[i].dl_handle;
     picd->ext_fn_ptr = _get_function_addr(dlh, NULL, "clGetExtensionFunctionAddress");
-    clIcdGetPlatformIDsKHR_fn plt_fn_ptr = 
+    clIcdGetPlatformIDsKHR_fn plt_fn_ptr =
       _get_function_addr(dlh, picd->ext_fn_ptr, "clIcdGetPlatformIDsKHR");
-    clGetPlatformInfo_fn plt_info_ptr = 
+    clGetPlatformInfo_fn plt_info_ptr =
       _get_function_addr(dlh, picd->ext_fn_ptr,	"clGetPlatformInfo");
     if( picd->ext_fn_ptr == NULL
 	|| plt_fn_ptr == NULL
@@ -354,7 +419,7 @@ static inline void _find_and_check_platforms(cl_uint num_icds) {
     }
     cl_uint num_valid_platforms=0;
     cl_uint j;
-    debug(D_LOG, "Try to load %d plateforms", num_platforms);
+    debug(D_LOG, "Try to load %d platforms", num_platforms);
     if (_allocate_platforms(num_platforms) < num_platforms) {
       free(platforms);
       debug(D_WARN, "Not enought platform allocated. Skipping ICD");
@@ -388,7 +453,7 @@ static inline void _find_and_check_platforms(cl_uint num_icds) {
 		      }
 		      debug(D_DUMP, "Supported extensions: %s", param_value);
 		      if( strstr(param_value, "cl_khr_icd") == NULL){
-			      free(param_value); 
+			      free(param_value);
 			      debug(D_WARN, "Missing khr extension in platform %i, skipping it", j);
 			      continue;
 		      }
@@ -435,12 +500,12 @@ static inline void _find_and_check_platforms(cl_uint num_icds) {
       dump_vendor_icd("after looking for platforms", &_icds[_num_icds]);
       _num_icds++;
       picd->num_platforms = num_valid_platforms;
-      _icds[i].first_platform = _num_picds - num_valid_platforms;
     } else {
       dlclose(dlh);
     }
     free(platforms);
   }
+  _sort_platforms(&_picds[0], _num_picds);
 }
 
 static void __initClIcd( void ) {
@@ -464,7 +529,7 @@ static void __initClIcd( void ) {
       is_dir=1;
     }
   }
-  
+
   if (!is_dir) {
     debug(D_LOG,"Only loading '%s' as an ICD", dir_path);
     num_icds = 1;
@@ -475,7 +540,7 @@ static void __initClIcd( void ) {
     if(dir == NULL) {
       if (errno == ENOTDIR) {
         debug(D_DUMP, "%s is not a directory, trying to use it as a ICD libname",
-  	dir_path);
+	  dir_path);
       }
       goto abort;
     }
@@ -490,7 +555,7 @@ static void __initClIcd( void ) {
   if (_icds == NULL) {
     goto abort;
   }
-  
+
   if (!is_dir) {
     if (_string_end_with_icd(dir_path)) {
       num_icds = 0;
@@ -544,9 +609,7 @@ static int gard=0;
 volatile static __thread int in_init = 0;
 volatile static cl_uint _initialized = 0;
 
-static inline void __attribute__((constructor)) _initClIcd( void ) {
-  if( _initialized )
-    return;
+static void _initClIcd_real( void ) {
 #ifdef USE_PTHREAD
   if (in_init) {
     /* probably reentrency, in_init is a __thread variable */
@@ -581,16 +644,23 @@ static inline void __attribute__((constructor)) _initClIcd( void ) {
   _initialized = 1;
 }
 
+static inline void _initClIcd( void ) {
+  if( __builtin_expect (_initialized, 1) )
+    return;
+  _initClIcd_real();
+}
+
 cl_platform_id __attribute__((visibility("internal")))
 getDefaultPlatformID() {
   static cl_platform_id defaultPlatformID=NULL;
   static int defaultSet=0;
+  _initClIcd();
   if (! defaultSet) {
     do {
       if(_num_picds == 0) {
 	break;
       }
-      const char *default_platform = getenv("OPENCL_ICD_DEFAULT_PLATFORM");
+      const char *default_platform = getenv("OCL_ICD_DEFAULT_PLATFORM");
       int num_default_platform;
       char *end_scan;
       if (! default_platform) {
@@ -624,7 +694,7 @@ typedef enum {
 
 static cl_int clGetICDLoaderInfoOCLICD(
   cl_icdl_info     param_name,
-  size_t           param_value_size, 
+  size_t           param_value_size,
   void *           param_value,
   size_t *         param_value_size_ret)
 {
@@ -817,21 +887,8 @@ clCreateContextFromType(const cl_context_properties *  properties ,
       i += 2;
     }
   } else {
-    const char *default_platform = getenv("OPENCL_ICD_DEFAULT_PLATFORM");
-    int num_default_platform;
-    char *end_scan;
-    if (! default_platform) {
-      num_default_platform = 0;
-    } else {
-      num_default_platform = strtol(default_platform, &end_scan, 10);
-      if (*default_platform == '\0' || *end_scan != '\0') {
-	goto out;
-      }
-    }
-    if (num_default_platform < 0 || num_default_platform >= _num_picds) {
-      goto out;
-    }
-    RETURN(_picds[num_default_platform].pid->dispatch->clCreateContextFromType
+    cl_platform_id default_platform=getDefaultPlatformID();
+    RETURN(default_platform->dispatch->clCreateContextFromType
 	(properties, device_type, pfn_notify, user_data, errcode_ret));
   }
  out:
